"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTabFromLocation = getTabFromLocation;
exports.TabManager = TabManager;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const react_router_dom_1 = require("react-router-dom");
const lodash_1 = require("lodash");
const TabLayout_1 = require("./TabLayout");
const TabContentLoading_1 = require("./TabContentLoading");
const ReadyContext_1 = require("../../routing/ReadyContext");
const defaultUrlParamName = 'tab';
/** Determines the current tab from the `Location` object. */
function getTabFromLocation(tabs, location, options) {
    if (tabs.length === 0)
        throw new Error('tabs array cannot be empty.');
    const { defaultTabName, urlParamName } = (0, lodash_1.defaults)(options, {
        urlParamName: defaultUrlParamName,
    });
    const searchParams = new URLSearchParams(location.search);
    const tabParam = searchParams.get(urlParamName);
    if (tabParam && tabs.some((t) => t[0] === tabParam))
        return tabParam;
    if (defaultTabName)
        return defaultTabName;
    return tabs[0][0];
}
// When the user switches to a tab that needs to load first, keep the height
// of the tab-content the same until the new tab finishes loading to avoid
// jarring changes in height
function useSmoothTabTransition(renderTabs, tab) {
    const tabContentRef = (0, react_1.useRef)(null);
    const [explicitTabContentHeight, setExplicitTabContentHeight] = (0, react_1.useState)();
    const [setHeightTo, setSetHeightTo] = (0, react_1.useState)();
    // When a new tab is about to mount, get the height of the tabContent
    // BEFORE the tabs actually switch
    function newTabWillMount() {
        if (tabContentRef.current) {
            const height = tabContentRef.current.offsetHeight;
            if (typeof height === 'number')
                setSetHeightTo(height);
        }
    }
    // Explicitly set the tabContent height immediately after the new tab becomes
    // visible. useLayoutEffect is essential here because we want the effect to
    // run *before the browser paints*.
    (0, react_1.useLayoutEffect)(() => {
        if (typeof setHeightTo === 'number') {
            setExplicitTabContentHeight(setHeightTo);
            setSetHeightTo(undefined);
        }
    }, [setHeightTo]);
    const currentRenderTab = renderTabs.find((rt) => rt[0] === tab);
    // Set explicit height to undefined when new tab becomes ready
    (0, react_1.useLayoutEffect)(() => {
        if (typeof explicitTabContentHeight === 'number') {
            if (currentRenderTab && currentRenderTab[1]) {
                setExplicitTabContentHeight(undefined);
            }
        }
    }, [explicitTabContentHeight, currentRenderTab]);
    return { tabContentRef, explicitTabContentHeight, newTabWillMount };
}
/**
 * A high-level component for creating user interfaces with tabs.
 *
 * - It stores the current tab name as a URL search params.
 * - It doesn't mount the a tab until the user actually visits that tab. (You can disable this with by setting `mountAllTabs` to true.)
 * - It shows a loading indicator if a tab needs to load data before it can render content.
 */
function TabManager({ tabs, children, defaultTabName, renderLoadingIndicator, mountAllTabs = false, urlParamName = defaultUrlParamName, displaySingleTab = true, className, tabContentClassName, }) {
    const navigate = (0, react_router_dom_1.useNavigate)();
    const { location } = (0, ReadyContext_1.useReadyCore)();
    let tab = '';
    if (tabs.length > 0) {
        tab = getTabFromLocation(tabs, location, { defaultTabName, urlParamName });
    }
    const [mountedTabs, setMountedTabs] = (0, react_1.useState)([tab]);
    const { tabContentRef, explicitTabContentHeight, newTabWillMount } = useSmoothTabTransition(children, tab);
    (0, react_1.useEffect)(() => {
        if (mountAllTabs)
            return;
        if (!mountedTabs.includes(tab)) {
            setMountedTabs((mountedTabs) => [...mountedTabs, tab]);
        }
    }, [mountedTabs, tab, mountAllTabs]);
    function onTabClick(tabName) {
        if (!mountedTabs.includes(tabName))
            newTabWillMount();
        const searchParams = new URLSearchParams(location.search);
        searchParams.set(urlParamName, tabName);
        navigate({
            ...location,
            search: searchParams.toString(),
        }, { replace: true });
    }
    const tabNames = mountAllTabs
        ? new Set(tabs.map((t) => t[0]))
        : new Set();
    function renderTab(theRenderTab) {
        const [thisTabName, ready, reactNode] = theRenderTab;
        if (!mountAllTabs && !mountedTabs.includes(thisTabName))
            return null;
        // We allow the `children` array to include tabs that are not in the
        // `tabs` array. This makes it simpler to implement tabs that are only
        // visible for users with certain permissions.
        if (mountAllTabs && !tabNames.has(thisTabName))
            return null;
        return ((0, jsx_runtime_1.jsxs)("div", { style: {
                display: tab === thisTabName ? undefined : 'none',
            }, className: !ready ? 'render-tab-loading' : undefined, children: [!ready && ((0, jsx_runtime_1.jsx)(TabContentLoading_1.TabContentLoading, { renderLoadingIndicator: renderLoadingIndicator })), (0, jsx_runtime_1.jsx)("div", { hidden: !ready, children: reactNode })] }, thisTabName));
    }
    if (tabs.length === 1 && !displaySingleTab) {
        if (!children || children.length === 0)
            return null;
        // Display contents without a tab or border
        return renderTab(children[0]);
    }
    return ((0, jsx_runtime_1.jsx)(TabLayout_1.TabLayout, { tabs: tabs, tab: tab, onTabClick: onTabClick, tabContentRef: tabContentRef, tabContentStyle: typeof explicitTabContentHeight === 'number'
            ? { height: explicitTabContentHeight }
            : undefined, className: className, tabContentClassName: tabContentClassName, children: children && children.map(renderTab) }));
}
