"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSelectStyles = void 0;
const color_1 = __importDefault(require("color"));
/** Styles the `react-select` component to match Bootstrap form-control inputs. */
const getSelectStyles = (options) => {
    const { valid, showValidation, themeColors, width, formControlSize, isMulti } = options;
    const disabledDarkenBy = 0.15;
    const indicatorPaddingSmY = '4px';
    const indicatorPaddingSmAll = `${indicatorPaddingSmY} 6px`;
    const multiValueMarginLgX = '4px';
    const noStyles = (base) => base;
    function getSvgStyles(defaultDim) {
        if (formControlSize === 'lg') {
            // Scale up SVG icons to match the larger control
            const scaleFactor = 1.25; // = $font-size-lg / $font-size-base
            const scaledDim = scaleFactor * defaultDim;
            return { width: scaledDim, height: scaledDim };
        }
        return {};
    }
    return {
        control: (base, props) => {
            const styles = { ...base };
            if (typeof width === 'number')
                styles.width = width;
            styles.borderColor = '#ced4da'; // $gray-400
            if (props.isDisabled) {
                styles.backgroundColor = '#e9ecef'; // $gray-200
            }
            else {
                styles.backgroundColor = 'white';
            }
            const primaryColor = new color_1.default(themeColors.primary);
            const dangerColor = new color_1.default(themeColors.danger);
            const successColor = new color_1.default(themeColors.success);
            if (showValidation) {
                const borderColor = valid ? successColor : dangerColor;
                styles.borderColor = borderColor.toString();
                styles['&:hover'].borderColor = borderColor.toString();
            }
            if (props.isFocused) {
                const borderColor = primaryColor.lighten(0.25);
                const boxShadowColor = primaryColor.alpha(0.25);
                styles.borderColor = borderColor.toString();
                styles['&:hover'].borderColor = borderColor.toString();
                styles.boxShadow = `0 0 0 0.2rem ${boxShadowColor.toString()}`;
            }
            if (formControlSize) {
                delete styles.minHeight;
                if (formControlSize === 'sm') {
                    styles.height = 'calc(1.8125rem + 2px)';
                    styles.fontSize = '0.875rem';
                    styles.lineHeight = '1.5';
                    styles.borderRadius = '0.2rem';
                }
                else if (formControlSize === 'lg') {
                    styles.height = 'calc(2.875rem + 2px)';
                    styles.fontSize = '1.25rem';
                    styles.lineHeight = '1.5';
                    styles.borderRadius = '0.3rem';
                }
            }
            return styles;
        },
        placeholder: (base, props) => {
            if (props.isDisabled)
                return base;
            return {
                ...base,
                color: themeColors.inputPlaceholder,
            };
        },
        dropdownIndicator: (base, props) => {
            const styles = { ...base };
            if (props.isDisabled) {
                styles.color = new color_1.default(base.color).darken(disabledDarkenBy).toString();
            }
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        clearIndicator: (base) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        loadingIndicator: (base) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        indicatorSeparator: (base, props) => {
            const styles = { ...base };
            if (!(props.hasValue && props.selectProps.isClearable)) {
                styles.display = 'none';
            }
            if (props.isDisabled) {
                styles.backgroundColor = new color_1.default(base.backgroundColor)
                    .darken(disabledDarkenBy)
                    .toString();
            }
            if (formControlSize === 'sm') {
                styles.marginTop = indicatorPaddingSmY;
                styles.marginBottom = indicatorPaddingSmY;
            }
            return styles;
        },
        menu: (base) => ({
            ...base,
            zIndex: 1000, // Value of $zindex-dropdown in the Bootstrap z-index master list
        }),
        valueContainer: (base, props) => {
            const styles = { ...base };
            if (formControlSize === 'sm') {
                styles.height = '1.8125rem';
                let paddingY = '0.25rem';
                if (props.isMulti)
                    paddingY = '0';
                // -2px because placeholder/option has 2px horiziontal margin
                styles.padding = `${paddingY} calc(0.5rem - 2px)`;
            }
            else if (formControlSize === 'lg') {
                styles.height = '2.875rem';
                let paddingY = '0.5rem';
                let optionMarginX = '2px';
                if (props.isMulti) {
                    paddingY = '0';
                    optionMarginX = multiValueMarginLgX;
                }
                // -2px because placeholder/option has 2px horiziontal margin
                styles.padding = `${paddingY} calc(1rem - ${optionMarginX})`;
            }
            return styles;
        },
        input: (base) => {
            const styles = { ...base };
            // Remove input's vertical margin and padding since we have already added padding to the valueContainer
            if (!isMulti && (formControlSize === 'sm' || formControlSize === 'lg')) {
                styles.marginTop = '0';
                styles.marginBottom = '0';
                styles.paddingTop = '0';
                styles.paddingBottom = '0';
            }
            return styles;
        },
        multiValue: (base, props) => {
            const styles = { ...base };
            if (formControlSize === 'lg') {
                styles.marginLeft = multiValueMarginLgX;
                styles.marginRight = multiValueMarginLgX;
            }
            if (props.data.isFixed)
                styles.backgroundColor = 'gray';
            return styles;
        },
        multiValueLabel: (base, props) => {
            const styles = { ...base };
            if (props.data.isFixed) {
                styles.color = 'white';
                styles.paddingRight = 6;
            }
            return styles;
        },
        multiValueRemove: (base, props) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(14);
            if (props.data.isFixed)
                styles.display = 'none';
            return styles;
        },
        singleValue: (base, props) => {
            const styles = { ...base };
            if (props.isDisabled) {
                styles.color = '#495057'; // $input-color == $gray-700
            }
            return styles;
        },
        /* Return a function for EVERY part of the select that can be styled. This way,
         * applications that use iti-react can write their own styling functions
         * that build on top of this one, like:
         *
         * return {
         *      control: (base, state) => {
         *          let styles = getSelectStyles(...).control(base, state)
         *
         *          // do stuff to styles
         *
         *          return styles
         *      }
         * }
         *
         * Then, in the future, we can add new styles here and any functions built on top of this one
         * will automatically utilize the new styles.
         */
        container: noStyles,
        group: noStyles,
        groupHeading: noStyles,
        indicatorsContainer: noStyles,
        loadingMessage: noStyles,
        menuList: noStyles,
        noOptionsMessage: noStyles,
        option: noStyles,
        menuPortal: noStyles,
    };
};
exports.getSelectStyles = getSelectStyles;
